@@ -35,16 +35,17 @@
 #include <stdlib.h>
 #include <string.h> /* memcpy */
 
-#include "simulation_data.h"
-#include "simulation/simulation_info_json.h"
-#include "util/omc_error.h"
-#include "util/varinfo.h"
+#include "../../simulation_data.h"
+#include "../simulation_info_json.h"
+#include "../../util/omc_error.h"
+#include "../../util/parallel_helper.h"
+#include "omc_math.h"
+#include "../../util/varinfo.h"
 #include "model_help.h"
 
 #include "linearSystem.h"
 #include "linearSolverTotalPivot.h"
 
-
 void debugMatrixDoubleLS(int logName, char* matrixName, double* matrix, int n, int m)
 {
   if(ACTIVE_STREAM(logName))
@@ -68,7 +69,7 @@ void debugMatrixDoubleLS(int logName, char* matrixName, double* matrix, int n, i
         }
         else
         {
-          sprintf(buffer, "%s%12.4g ", buffer, matrix[i + j*(m-1)]);
+          sprintf(buffer, "%s %12.4g", buffer, matrix[i + j*(m-1)]);
         }
       }
       infoStreamPrint(logName, 0, "%s", buffer);
@@ -87,14 +88,20 @@ void debugVectorDoubleLS(int logName, char* vectorName, double* vector, int n)
 
     infoStreamPrint(logName, 1, "%s [%d-dim]", vectorName, n);
     buffer[0] = 0;
-    for(i=0; i<n;i++)
+    if (vector[0]<-1e+300)
+      sprintf(buffer, "%s -INF", buffer);
+    else if (vector[0]>1e+300)
+      sprintf(buffer, "%s +INF", buffer);
+    else
+      sprintf(buffer, "%s %16.8g", buffer, vector[0]);
+    for(i=1; i<n;i++)
     {
       if (vector[i]<-1e+300)
-        sprintf(buffer, "%s -INF ", buffer);
+        sprintf(buffer, "%s -INF", buffer);
       else if (vector[i]>1e+300)
-        sprintf(buffer, "%s +INF ", buffer);
+        sprintf(buffer, "%s +INF", buffer);
       else
-        sprintf(buffer, "%s%16.8g ", buffer, vector[i]);
+        sprintf(buffer, "%s %16.8g", buffer, vector[i]);
     }
     infoStreamPrint(logName, 0, "%s", buffer);
     free(buffer);
@@ -177,17 +184,21 @@ void getIndicesOfPivotElementLS(int *n, int *m, int *l, double* A, int *indRow,
   }
 }
 
-/*! \ linear solver for A*x = b based on a total pivot search
- *  \ input matrix Ab: first n columns are matrix A
- *                     last column is -b
- *    output x: solution dim n+1, last column is 1 for solvable systems!
- *           rank: rank of matrix Ab
- *           det: determinant of matrix A
+/**
+ * @brief Linear solver for A*x = b based on a total pivot search.
  *
- *  \author bbachmann
+ * \author bbachmann
+ *
+ * @param data    Simulation data.
+ * @param n       Size of matrix a
+ * @param x       On return: Solution dim n+1, last column is 1 for solvable systems.
+ * @param Ab      Matrix A|b: first n columns are matrix A, last column is -b
+ * @param indRow  Work array for row indices, used for coloring.
+ * @param indCol  Work array for column indices, used for coloring.
+ * @param rank    On return: Rank of matrix A|b.
+ * @return int    Return 0 on success, -1 if system is under-determined.
  */
-
-int solveSystemWithTotalPivotSearchLS(int n, double* x, double* A, int* indRow, int* indCol, int *rank)
+int solveSystemWithTotalPivotSearchLS(DATA* data, int n, double* x, double* Ab, int* indRow, int* indCol, int *rank)
 {
    int i, k, j, l, m=n+1, nrsh=1, singular=0;
    int pCol, pRow;
@@ -197,7 +208,7 @@ int solveSystemWithTotalPivotSearchLS(int n, double* x, double* A, int* indRow,
    int r,s;
    int permutation = 1;
 
-   /* assume full rank of matrix [n x (n+1)] */
+   /* assume full rank of matrix A|b [n x (n+1)] */
    *rank = n;
 
    for (i=0; i<n; i++) {
@@ -208,12 +219,18 @@ int solveSystemWithTotalPivotSearchLS(int n, double* x, double* A, int* indRow,
    }
 
    for (i = 0; i < n; i++) {
-    getIndicesOfPivotElementLS(&n, &n, &i, A, indRow, indCol, &pRow, &pCol, &absMax);
+    getIndicesOfPivotElementLS(&n, &n, &i, Ab, indRow, indCol, &pRow, &pCol, &absMax);
     /* this criteria should be evaluated and may be improved in future */
-    if (absMax<DBL_EPSILON) {
+    if (absMax < DBL_EPSILON) {
       *rank = i;
-      warningStreamPrint(LOG_LS, 0, "Matrix singular!");
-      debugIntLS(LOG_LS,"rank = ", *rank);
+      if (data->simulationInfo->initial) {
+        warningStreamPrint(LOG_LS, 1, "Total Pivot: Matrix (nearly) singular at initialization.");
+      } else {
+        warningStreamPrint(LOG_LS, 1, "Total Pivot: Matrix (nearly) singular at time %f.", data->localData[0]->timeValue);
+      }
+      warningStreamPrint(LOG_LS, 0, "Continuing anyway. For more information please use -lv %s.", LOG_STREAM_NAME[LOG_LS]);
+      messageCloseWarning(LOG_LS);
+      infoStreamPrint(LOG_LS, 0, "rank =  %u", *rank);
       break;
     }
     /* swap row indices */
@@ -231,31 +248,31 @@ int solveSystemWithTotalPivotSearchLS(int n, double* x, double* A, int* indRow,
 
     /* Gauss elimination of row indRow[i] */
     for (k=i+1; k<n; k++) {
-      hValue = -A[indRow[k] + indCol[i]*n]/A[indRow[i] + indCol[i]*n];
+      hValue = -Ab[indRow[k] + indCol[i]*n]/Ab[indRow[i] + indCol[i]*n];
       for (j=i+1; j<m; j++) {
-        A[indRow[k] + indCol[j]*n] = A[indRow[k] + indCol[j]*n] + hValue*A[indRow[i] + indCol[j]*n];
+        Ab[indRow[k] + indCol[j]*n] = Ab[indRow[k] + indCol[j]*n] + hValue*Ab[indRow[i] + indCol[j]*n];
       }
-      A[indRow[k] + indCol[i]*n] = 0;
+      Ab[indRow[k] + indCol[i]*n] = 0;
     }
   }
 
-  debugMatrixDoubleLS(LOG_LS_V,"LGS: matrix Ab manipulated",A, n, n+1);
-  /* solve even singular matrix !!! */
+  debugMatrixDoubleLS(LOG_LS_V,"LGS: matrix Ab manipulated",Ab, n, n+1);
+  /* solve even singular matrix */
   for (i=n-1;i>=0; i--) {
     if (i>=*rank) {
       /* this criteria should be evaluated and may be improved in future */
-      if (fabs(A[indRow[i] + n*n])>1e-12) {
+      if (fabs(Ab[indRow[i] + n*n])>1e-12) {
         warningStreamPrint(LOG_LS, 0, "under-determined linear system not solvable!");
         return -1;
       } else {
         x[indCol[i]] = 0.0;
       }
     } else {
-      x[indCol[i]] = -A[indRow[i] + n*n];
+      x[indCol[i]] = -Ab[indRow[i] + n*n];
       for (j=n-1; j>i; j--) {
-        x[indCol[i]] = x[indCol[i]] - A[indRow[i] + indCol[j]*n]*x[indCol[j]];
+        x[indCol[i]] = x[indCol[i]] - Ab[indRow[i] + indCol[j]*n]*x[indCol[j]];
       }
-      x[indCol[i]]=x[indCol[i]]/A[indRow[i] + indCol[i]*n];
+      x[indCol[i]]=x[indCol[i]]/Ab[indRow[i] + indCol[i]*n];
     }
   }
   x[n]=1.0;
@@ -303,7 +320,7 @@ int freeTotalPivotData(void** voiddata)
   free(data->indCol);
 
   free(voiddata[1]);
-  voiddata[1] = 0;
+  voiddata[1] = NULL;
 
   return 0;
 }
@@ -320,37 +337,39 @@ int freeTotalPivotData(void** voiddata)
  */
 int getAnalyticalJacobianTotalPivot(DATA* data, threadData_t *threadData, double* jac, int sysNumber)
 {
-  int i,j,k,l,ii,currentSys = sysNumber;
-  LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[currentSys]);
+  int i,j,k,l,ii;
+  LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[sysNumber]);
 
   const int index = systemData->jacobianIndex;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
+
+  ANALYTIC_JACOBIAN* jacobian = systemData->parDynamicData[omc_get_thread_num()].jacobian;
+  ANALYTIC_JACOBIAN* parentJacobian = systemData->parDynamicData[omc_get_thread_num()].parentJacobian;
 
   memset(jac, 0, (systemData->size)*(systemData->size)*sizeof(double));
 
-  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
+  for(i=0; i < jacobian->sparsePattern->maxColors; i++)
   {
     /* activate seed variable for the corresponding color */
     for(ii=0; ii < jacobian->sizeCols; ii++)
-      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
         jacobian->seedVars[ii] = 1;
 
-    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, systemData->parentJacobian);
+    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, parentJacobian);
 
     for(j = 0; j < jacobian->sizeCols; j++)
     {
       if(jacobian->seedVars[j] == 1)
       {
-        ii = jacobian->sparsePattern.leadindex[j];
-        while(ii < jacobian->sparsePattern.leadindex[j+1]) {
-          l  = jacobian->sparsePattern.index[ii];
+        ii = jacobian->sparsePattern->leadindex[j];
+        while(ii < jacobian->sparsePattern->leadindex[j+1]) {
+          l  = jacobian->sparsePattern->index[ii];
           k  = j*jacobian->sizeRows + l;
           jac[k] = jacobian->resultVars[l];
           ii++;
         }
       }
       /* de-activate seed variable for the corresponding color */
-      if(jacobian->sparsePattern.colorCols[j]-1 == i) {
+      if(jacobian->sparsePattern->colorCols[j]-1 == i) {
         jacobian->seedVars[j] = 0;
       }
     }
@@ -364,36 +383,40 @@ int getAnalyticalJacobianTotalPivot(DATA* data, threadData_t *threadData, double
  *
  *
  */
-static int wrapper_fvec_totalpivot(double* x, double* f, void** data, int sysNumber)
+static int wrapper_fvec_totalpivot(double* x, double* f, RESIDUAL_USERDATA* resUserData, int sysNumber)
 {
   int currentSys = sysNumber;
   int iflag = 0;
-  /* NONLINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->nonlinearSystemData[currentSys]); */
-  /* DATA_NEWTON* solverData = (DATA_NEWTON*)(systemData->solverData); */
 
-  (*((DATA*)data[0])->simulationInfo->linearSystemData[currentSys].residualFunc)(data, x, f, &iflag);
+  resUserData->data->simulationInfo->linearSystemData[currentSys].residualFunc(resUserData, x, f, &iflag);
   return 0;
 }
 
-/*! \fn solve linear system with totalpivot method
+/**
+ * @brief Solve linear system with total pivot method.
  *
- *  \param [in]  [data]
- *                [sysNumber] index of the corresponing non-linear system
+ * \author bbachmann
  *
- *  \author bbachmann
+ * @param data        Runtime data struct.
+ * @param threadData  Thread data for error handling.
+ * @param sysNumber   Index of the corresponding non-linear system.
+ * @param aux_x       Work array with old values of x. Will be overwritten with solution.
+ * @return int        Return 1 on success and 0 on failure.
  */
 int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
-  void *dataAndThreadData[2] = {data, threadData};
+  RESIDUAL_USERDATA resUserData = {.data=data, .threadData=threadData, .solverData=NULL};
   int i, j;
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_TOTALPIVOT* solverData = (DATA_TOTALPIVOT*) systemData->solverData[1];
+  DATA_TOTALPIVOT* solverData = (DATA_TOTALPIVOT*) systemData->parDynamicData[omc_get_thread_num()].solverData[1];
+
   int n = systemData->size, status;
   double fdeps = 1e-8;
   double xTol = 1e-8;
   int eqSystemNumber = systemData->equationIndex;
   int indexes[2] = {1,eqSystemNumber};
   int rank;
+  _omc_scalar residualNorm = 0;
 
   /* We are given the number of the linear system.
    * We want to look it up among all equations. */
@@ -401,27 +424,26 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber, double*
   int success = 1;
   double tmpJacEvalTime;
 
-  infoStreamPrintWithEquationIndexes(LOG_LS, 0, indexes, "Start solving Linear System %d (size %d) at time %g with Total Pivot Solver",
-         eqSystemNumber, (int) systemData->size,
-         data->localData[0]->timeValue);
+  infoStreamPrintWithEquationIndexes(LOG_LS, omc_dummyFileInfo, 0, indexes, "Start solving Linear System %d (size %d) at time %g with Total Pivot Solver",
+                                     eqSystemNumber, (int) systemData->size,
+                                     data->localData[0]->timeValue);
 
   debugVectorDoubleLS(LOG_LS_V,"SCALING",systemData->nominal,n);
   debugVectorDoubleLS(LOG_LS_V,"Old VALUES",aux_x,n);
 
   rt_ext_tp_tick(&(solverData->timeClock));
-  if (0 == systemData->method){
+  if (0 == systemData->method) {
 
     /* reset matrix A */
-    vecConstLS(n*n, 0.0, systemData->A);
+    vecConstLS(n*n, 0.0, systemData->parDynamicData[omc_get_thread_num()].A);
     /* update matrix A -> first n columns of matrix Ab*/
     systemData->setA(data, threadData, systemData);
-    vecCopyLS(n*n, systemData->A, solverData->Ab);
+    vecCopyLS(n*n, systemData->parDynamicData[omc_get_thread_num()].A, solverData->Ab);
 
     /* update vector b (rhs) -> -b is last column of matrix Ab*/
     rt_ext_tp_tick(&(solverData->timeClock));
     systemData->setb(data, threadData, systemData);
-    vecScalarMultLS(n, systemData->b, -1.0, solverData->Ab + n*n);
-
+    vecScalarMultLS(n, systemData->parDynamicData[omc_get_thread_num()].b, -1.0, solverData->Ab + n*n);
   } else {
 
     /* calculate jacobian -> first n columns of matrix Ab*/
@@ -431,7 +453,7 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber, double*
       assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
     }
     /* calculate vector b (rhs) -> -b is last column of matrix Ab */
-    wrapper_fvec_totalpivot(aux_x, solverData->Ab + n*n, dataAndThreadData, sysNumber);
+    wrapper_fvec_totalpivot(aux_x, solverData->Ab + n*n, &resUserData, sysNumber);
   }
   tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
   systemData->jacobianTime += tmpJacEvalTime;
@@ -439,27 +461,35 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber, double*
   debugMatrixDoubleLS(LOG_LS_V,"LGS: matrix Ab",solverData->Ab, n, n+1);
 
   rt_ext_tp_tick(&(solverData->timeClock));
-  status = solveSystemWithTotalPivotSearchLS(n, solverData->x, solverData->Ab, solverData->indRow, solverData->indCol, &rank);
+  status = solveSystemWithTotalPivotSearchLS(data, n, solverData->x, solverData->Ab, solverData->indRow, solverData->indCol, &rank);
   infoStreamPrint(LOG_LS_V, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
 
-  if (status != 0)
-  {
+  if (status != 0) {
+    // ToDo Rework stream prints like this one to work in parallel regions
+#ifdef USE_PARJAC
+    warningStreamPrint(LOG_STDOUT, 0, "Thread %u: Error solving linear system of equations (no. %d) at time %f.", omc_get_thread_num(), (int)systemData->equationIndex, data->localData[0]->timeValue);
+    success = 0;
+#else
     warningStreamPrint(LOG_STDOUT, 0, "Error solving linear system of equations (no. %d) at time %f.", (int)systemData->equationIndex, data->localData[0]->timeValue);
     success = 0;
+#endif
   } else {
-
     debugVectorDoubleLS(LOG_LS_V, "SOLUTION:", solverData->x, n+1);
-    if (1 == systemData->method){
+    if (1 == systemData->method) {
       /* add the solution to old solution vector*/
       vecAddLS(n, aux_x, solverData->x, aux_x);
-      wrapper_fvec_totalpivot(aux_x, solverData->b, dataAndThreadData, sysNumber);
+      wrapper_fvec_totalpivot(aux_x, solverData->b, &resUserData, sysNumber);
     } else {
        /* take the solution */
        vecCopyLS(n, solverData->x, aux_x);
     }
 
-    if (ACTIVE_STREAM(LOG_LS_V)){
-      infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+    if (ACTIVE_STREAM(LOG_LS_V)) {
+      if (1 == systemData->method) {
+        infoStreamPrint(LOG_LS_V, 1, "Residual Norm %.15g of solution x:", residualNorm);
+      } else {
+        infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+      }
       infoStreamPrint(LOG_LS_V, 0, "System %d numVars %d.", eqSystemNumber, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).numVar);
       for(i=0; i<systemData->size; ++i)
       {
