@@ -517,6 +517,7 @@ void Arkode::ArkodeCore()
     }
 }
 void Arkode::setTimeOut(unsigned int time_out)
+
   {
        SimulationMonitor::setTimeOut(time_out);
   }
@@ -558,7 +559,7 @@ void Arkode::writeArkodeOutput(const double &time, const double &h, const int &s
       {
         _time_system->setTime(time);
         _continuous_system->setContinuousStates(_z);
-        _continuous_system->setRHS(oldValues);
+        _continuous_system->setStateDerivatives(oldValues);
         delete[] oldValues;
         //_continuous_system->evaluateAll(IContinuous::CONTINUOUS);
       }
