@@ -39,6 +39,27 @@
 
 #include <QRadioButton>
 
+class FinalEachToolButton : public QToolButton
+{
+  Q_OBJECT
+public:
+  FinalEachToolButton(bool canHaveEach, QWidget *parent = nullptr);
+
+  void setFinal(bool final);
+  bool isFinal() const {return mpFinalAction->isChecked();}
+  void setEach(bool each);
+  bool isEach() const {return mpEachAction->isChecked();}
+  bool isModified() const;
+private:
+  QMenu *mpFinalEachMenu;
+  QAction *mpFinalAction;
+  bool mFinalDefault = false;
+  QAction *mpEachAction;
+  bool mEachDefault = false;
+public slots:
+  void showParameterMenu();
+};
+
 class ElementParametersOld;
 class ElementParameters;
 class Parameter : public QObject
@@ -72,9 +93,11 @@ public:
   bool isShowStartAndFixed() const {return mShowStartAndFixed;}
   const StringAnnotation &getGroupImage() const {return mGroupImage;}
   void updateNameLabel();
+  QString getName() const {return mName;}
   Label* getNameLabel() {return mpNameLabel;}
   FixedCheckBox* getFixedCheckBox() {return mpFixedCheckBox;}
-  QString getOriginalFixedValue() {return mOriginalFixedValue;}
+  QString getOriginalFixedValue() const {return mOriginalFixedValue;}
+  FinalEachToolButton *getFixedFinalEachMenu() const {return mpFixedFinalEachMenuButton;}
   void setValueType(ValueType valueType) {mValueType = valueType;}
   void setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified = false, bool adjustSize = true, bool unitComboBoxChanged = false);
   ValueType getValueType() {return mValueType;}
@@ -82,6 +105,7 @@ public:
   bool isValueModified();
   QString getValue();
   QToolButton *getEditRedeclareClassButton() const {return mpEditRedeclareClassButton;}
+  FinalEachToolButton *getFinalEachMenu() const {return mpFinalEachMenuButton;}
   QToolButton* getFileSelectorButton() {return mpFileSelectorButton;}
   void setLoadSelectorFilter(QString loadSelectorFilter) {mLoadSelectorFilter = loadSelectorFilter;}
   QString getLoadSelectorFilter() {return mLoadSelectorFilter;}
@@ -91,10 +115,13 @@ public:
   QString getSaveSelectorFilter() {return mSaveSelectorFilter;}
   void setSaveSelectorCaption(QString saveSelectorCaption) {mSaveSelectorCaption = saveSelectorCaption;}
   QString getSaveSelectorCaption() {return mSaveSelectorCaption;}
+  void setHasDisplayUnit(bool hasDisplayUnit) {mHasDisplayUnit = hasDisplayUnit;}
+  bool hasDisplayUnit() const {return mHasDisplayUnit;}
   QString getUnit() {return mUnit;}
   void setDisplayUnit(QString displayUnit) {mDisplayUnit = displayUnit;}
   QString getDisplayUnit() {return mDisplayUnit;}
   QComboBox* getUnitComboBox() {return mpUnitComboBox;}
+  FinalEachToolButton *getDisplayUnitFinalEachMenu() const {return mpDisplayUnitFinalEachMenuButton;}
   Label* getCommentLabel() {return mpCommentLabel;}
   void setFixedState(QString fixed, bool defaultValue);
   QString getFixedState() const;
@@ -119,9 +146,11 @@ private:
   StringAnnotation mGroupImage;
   BooleanAnnotation mConnectorSizing;
 
+  QString mName;
   Label *mpNameLabel;
   FixedCheckBox *mpFixedCheckBox;
   QString mOriginalFixedValue;
+  FinalEachToolButton *mpFixedFinalEachMenuButton = 0;
   ValueType mValueType;
   bool mValueCheckBoxModified;
   QString mDefaultValue;
@@ -129,27 +158,33 @@ private:
   QLineEdit *mpValueTextBox;
   QCheckBox *mpValueCheckBox;
   QToolButton *mpEditRedeclareClassButton = 0;
+  FinalEachToolButton *mpFinalEachMenuButton = 0;
   QToolButton *mpFileSelectorButton;
+  bool mHasDisplayUnit = false;
   QString mUnit;
   QString mDisplayUnit;
   QString mPreviousUnit;
   QComboBox *mpUnitComboBox;
+  FinalEachToolButton *mpDisplayUnitFinalEachMenuButton = 0;
   Label *mpCommentLabel;
 
   void createValueWidget();
   void enableDisableUnitComboBox(const QString &value);
   void updateValueBinding(const FlatModelica::Expression expression);
+  bool isValueModifiedHelper() const;
 public slots:
   void editRedeclareClassButtonClicked();
   void fileSelectorButtonClicked();
   void unitComboBoxChanged(int index);
   void valueComboBoxChanged(int index);
   void valueCheckBoxChanged(bool toggle);
-  void valueTextBoxEdited(const QString &text);
   void showFixedMenu();
   void trueFixedClicked();
   void falseFixedClicked();
   void inheritedFixedClicked();
+  // QObject interface
+public:
+  virtual bool eventFilter(QObject *pWatched, QEvent *pEvent) override;
 };
 
 class GroupBox : public QGroupBox
@@ -186,23 +221,23 @@ class ElementParameters : public QDialog
 {
   Q_OBJECT
 public:
-  ElementParameters(ModelInstance::Element *pElement, GraphicsView *pGraphicsView, bool inherited, bool nested, const ModelInstance::Modifier defaultElementModifier,
-                    const ModelInstance::Modifier replaceableConstrainedByModifier, const ModelInstance::Modifier elementModifier, QWidget *pParent = 0);
+  ElementParameters(ModelInstance::Element *pElement, GraphicsView *pGraphicsView, bool inherited, bool nested, ModelInstance::Modifier *pDefaultElementModifier,
+                    ModelInstance::Modifier *pReplaceableConstrainedByModifier, ModelInstance::Modifier *pElementModifier, QWidget *pParent = 0);
   ~ElementParameters();
   QString getElementParentClassName() const;
   GraphicsView *getGraphicsView() const {return mpGraphicsView;}
   bool isInherited() const {return mInherited;}
   QString getModification() const {return mModification;}
-  void applyFinalStartFixedAndDisplayUnitModifiers(Parameter *pParameter, const ModelInstance::Modifier &modifier, bool defaultValue, bool isElementModification);
+  void applyFinalStartFixedAndDisplayUnitModifiers(Parameter *pParameter, ModelInstance::Modifier *pModifier, bool defaultValue, bool isElementModification, bool checkFinal);
   void updateParameters();
 private:
   ModelInstance::Element *mpElement;
   GraphicsView *mpGraphicsView;
   bool mInherited;
   bool mNested;
-  ModelInstance::Modifier mDefaultElementModifier;
-  ModelInstance::Modifier mReplaceableConstrainedByModifier;
-  ModelInstance::Modifier mElementModifier;
+  ModelInstance::Modifier *mpDefaultElementModifier;
+  ModelInstance::Modifier *mpReplaceableConstrainedByModifier;
+  ModelInstance::Modifier *mpElementModifier;
   QString mModification;
   Label *mpParametersHeading;
   QFrame *mHorizontalLine;
@@ -230,7 +265,7 @@ private:
   void fetchElementExtendsModifiers(ModelInstance::Model *pModelInstance);
   void fetchElementModifiers();
   void fetchClassExtendsModifiers(ModelInstance::Element *pModelElement);
-  void applyModifiers(const ModelInstance::Modifier modifiers, bool defaultValue);
+  void applyModifier(ModelInstance::Modifier *pModifier, bool defaultValue);
   Parameter* findParameter(LibraryTreeItem *pLibraryTreeItem, const QString &parameter, Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
   Parameter* findParameter(const QString &parameter, Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
 public slots:
