@@ -496,6 +496,8 @@ namespace ModelInstance
     : mPlacementAnnotation(pParentModel)
   {
     mpParentModel = pParentModel;
+    mpIconAnnotation = std::make_unique<IconDiagramAnnotation>(mpParentModel);
+    mpDiagramAnnotation = std::make_unique<IconDiagramAnnotation>(mpParentModel);
     mDocumentationClass = false;
     mVersion = "";
     mVersionDate = "";
@@ -519,12 +521,10 @@ namespace ModelInstance
   void Annotation::deserialize(const QJsonObject &jsonObject)
   {
     if (jsonObject.contains("Icon")) {
-      mpIconAnnotation = std::make_unique<IconDiagramAnnotation>(mpParentModel);
       mpIconAnnotation->deserialize(jsonObject.value("Icon").toObject());
     }
 
     if (jsonObject.contains("Diagram")) {
-      mpDiagramAnnotation = std::make_unique<IconDiagramAnnotation>(mpParentModel);
       mpDiagramAnnotation->deserialize(jsonObject.value("Diagram").toObject());
     }
 
@@ -608,18 +608,6 @@ namespace ModelInstance
     }
   }
 
-  IconDiagramAnnotation *Annotation::getIconAnnotation() const
-  {
-    return mpIconAnnotation ? mpIconAnnotation.get() : &IconDiagramAnnotation::defaultIconDiagramAnnotation;
-  }
-
-  IconDiagramAnnotation *Annotation::getDiagramAnnotation() const
-  {
-    return mpDiagramAnnotation ? mpDiagramAnnotation.get() : &IconDiagramAnnotation::defaultIconDiagramAnnotation;
-  }
-
-  IconDiagramAnnotation IconDiagramAnnotation::defaultIconDiagramAnnotation{nullptr};
-
   IconDiagramAnnotation::IconDiagramAnnotation(Model *pParentModel)
   {
     mpParentModel = pParentModel;
@@ -1094,7 +1082,6 @@ namespace ModelInstance
     }
 
     if (mModelJson.contains("annotation")) {
-      mpAnnotation = std::make_unique<Annotation>(this);
       mpAnnotation->deserialize(mModelJson.value("annotation").toObject());
     }
 
@@ -1197,7 +1184,7 @@ namespace ModelInstance
 
   bool Model::isEnumeration() const
   {
-    return (mRestriction.compare(QStringLiteral("enumeration")) == 0);
+    return getRootType() == QLatin1String("enumeration");
   }
 
   bool Model::isType() const
@@ -1488,14 +1475,37 @@ namespace ModelInstance
 
   FlatModelica::Expression* Model::getVariableBinding(const QString &variableName)
   {
+    QString curName;
+    bool last;
+
+    if (variableName.contains("."))
+    {
+      curName = StringHandler::getFirstWordBeforeDot(variableName);
+      last = false;
+    }
+    else
+    {
+      curName = variableName;
+      last = true;
+    }
+
     foreach (auto pElement, mElements) {
       if (pElement->isComponent()) {
-        if (pElement->getName().compare(variableName) == 0) {
-          return &pElement->getBinding();
+        if (pElement->getName().compare(curName) == 0) {
+          if (last) {
+            return &pElement->getBinding();
+          } else {
+            if (!pElement->getModel()) {
+              return nullptr;
+            }
+            return pElement->getModel()->getVariableBinding(StringHandler::removeFirstWordAfterDot(variableName));
+          }
         }
       } else if (pElement->isExtend() && pElement->getModel()) {
         auto expression = pElement->getModel()->getVariableBinding(variableName);
-        if (expression) return expression;
+        if (expression) {
+          return expression;
+        }
       }
     }
 
@@ -1552,6 +1562,7 @@ namespace ModelInstance
     mConnections.clear();
     mTransitions.clear();
     mInitialStates.clear();
+    mpAnnotation = std::make_unique<Annotation>(this);
   }
 
   Transformation::Transformation()
@@ -1749,11 +1760,6 @@ namespace ModelInstance
       mComment = jsonObject.value("comment").toString();
     }
 
-    if (jsonObject.contains("annotation")) {
-      mpAnnotation = std::make_unique<Annotation>(mpParentModel);
-      mpAnnotation->deserialize(jsonObject.value("annotation").toObject());
-    }
-
     deserialize_impl(jsonObject);
   }
 
@@ -1943,6 +1949,12 @@ namespace ModelInstance
         mBaseClass = mpModel->getName();
       }
     }
+
+    // Always create Annotation for extend element. See #11363
+    mpAnnotation = std::make_unique<Annotation>(mpParentModel);
+    if (jsonObject.contains("annotation")) {
+      mpAnnotation->deserialize(jsonObject.value("annotation").toObject());
+    }
   }
 
   /*!
@@ -2033,6 +2045,11 @@ namespace ModelInstance
       mpPrefixes = std::make_unique<Prefixes>(mpParentModel);
       mpPrefixes->deserialize(jsonObject.value("prefixes").toObject());
     }
+
+    if (jsonObject.contains("annotation")) {
+      mpAnnotation = std::make_unique<Annotation>(mpParentModel);
+      mpAnnotation->deserialize(jsonObject.value("annotation").toObject());
+    }
   }
 
   /*!
@@ -2043,7 +2060,12 @@ namespace ModelInstance
   QString Component::getQualifiedName() const
   {
     if (mpParentModel && mpParentModel->getParentElement()) {
-      return mpParentModel->getParentElement()->getQualifiedName() % "." % mName;
+      QString name = mpParentModel->getParentElement()->getQualifiedName();
+      if (name.isEmpty()) {
+        return mName;
+      } else {
+        return name % "." % mName;
+      }
     } else {
       return mName;
     }
@@ -2115,12 +2137,22 @@ namespace ModelInstance
     if (jsonObject.contains("source")) {
       mSource.deserialize(jsonObject.value("source").toObject());
     }
+
+    if (jsonObject.contains("annotation")) {
+      mpAnnotation = std::make_unique<Annotation>(mpParentModel);
+      mpAnnotation->deserialize(jsonObject.value("annotation").toObject());
+    }
   }
 
   QString ReplaceableClass::getQualifiedName() const
   {
     if (mpParentModel && mpParentModel->getParentElement()) {
-      return mpParentModel->getParentElement()->getQualifiedName() % "." % mName;
+      QString name = mpParentModel->getParentElement()->getQualifiedName();
+      if (name.isEmpty()) {
+        return mName;
+      } else {
+        return name % "." % mName;
+      }
     } else {
       return mName;
     }
