@@ -65,7 +65,7 @@ public:
   SimulationDialog(QWidget *pParent = 0);
   ~SimulationDialog();
   void show(LibraryTreeItem *pLibraryTreeItem, bool isReSimulate, SimulationOptions simulationOptions);
-  void directSimulate(LibraryTreeItem *pLibraryTreeItem, bool launchTransformationalDebugger, bool launchAlgorithmicDebugger, bool launchAnimation);
+  void directSimulate(LibraryTreeItem *pLibraryTreeItem, bool launchTransformationalDebugger, bool launchAlgorithmicDebugger, bool launchAnimation, bool enableDataReconciliation);
   OpcUaClient* getOpcUaClient(int port);
   void removeSimulationOutputWidget(SimulationOutputWidget* pSimulationOutputWidget);
 private:
@@ -92,7 +92,7 @@ private:
   QLineEdit *mpToleranceTextBox;
   Label *mpJacobianLabel;
   QComboBox *mpJacobianComboBox;
-  QGroupBox *mpDasslIdaOptionsGroupBox;
+  QGroupBox *mpOptionsGroupBox;
   QCheckBox *mpRootFindingCheckBox;
   QCheckBox *mpRestartAfterEventCheckBox;
   Label *mpInitialStepSizeLabel;
@@ -166,23 +166,12 @@ private:
   QLineEdit *mpResultFileNameTextBox;
   Label *mpVariableFilterLabel;
   QLineEdit *mpVariableFilterTextBox;
+  QToolButton *mpVariableFilterHelpButton;
   QCheckBox *mpProtectedVariablesCheckBox;
+  QCheckBox *mpIgnoreHideResultCheckBox;
   QCheckBox *mpEquidistantTimeGridCheckBox;
   QCheckBox *mpStoreVariablesAtEventsCheckBox;
   QCheckBox *mpShowGeneratedFilesCheckBox;
-  // Data Reconciliation Tab
-  QWidget *mpDataReconciliationTab;
-  QGroupBox *mpDataReconciliationGroupBox;
-  Label *mpDataReconciliationAlgorithmLabel;
-  QComboBox *mpDataReconciliationAlgorithmComboBox;
-  Label *mpDataReconciliationMeasurementInputFileLabel;
-  QLineEdit *mpDataReconciliationMeasurementInputFileTextBox;
-  QPushButton *mpDataReconciliationMeasurementInputFileBrowseButton;
-  Label *mpDataReconciliationCorrelationMatrixInputFileLabel;
-  QLineEdit *mpDataReconciliationCorrelationMatrixInputFileTextBox;
-  QPushButton *mpDataReconciliationCorrelationMatrixInputFileBrowseButton;
-  Label *mpDataReconciliationEpsilonLabel;
-  QLineEdit *mpDataReconciliationEpsilonTextBox;
   // checkboxes
   QCheckBox *mpSaveExperimentAnnotationCheckBox;
   QCheckBox *mpSaveSimulationFlagsAnnotationCheckBox;
@@ -206,12 +195,12 @@ private:
   void applySimulationOptions(SimulationOptions simulationOptions);
   bool translateModel(QString simulationParameters);
   SimulationOptions createSimulationOptions();
-  void createAndShowSimulationOutputWidget(SimulationOptions simulationOptions);
+  void createAndShowSimulationOutputWidget(const SimulationOptions &simulationOptions);
   void showSimulationOutputWidget(SimulationOutputWidget *pSimulationOutputWidget);
   void saveExperimentAnnotation();
   void saveSimulationFlagsAnnotation();
   void saveTranslationFlagsAnnotation();
-  void performSimulation();
+  void performSimulation(const SimulationOptions &simulationOptions);
   void saveDialogGeometry();
   void killSimulationProcess(int port);
   void removeVariablesFromTree(QString className);
@@ -224,10 +213,8 @@ public:
 public slots:
   void numberOfIntervalsRadioToggled(bool toggle);
   void intervalRadioToggled(bool toggle);
-  void updateMethodToolTip(int index);
-  void enableDasslIdaOptions(QString method);
+  void enableDisableOptions(QString method);
   void showIntegrationHelp();
-  void updateJacobianToolTip(int index);
   void buildOnly(bool checked);
   void interactiveSimulation(bool checked);
   void browseModelSetupFile();
@@ -239,10 +226,48 @@ public slots:
   void updateYAxis(double min, double max);
 private slots:
   void resultFileNameChanged(QString text);
-  void browseDataReconciliationMeasurementInputFile();
-  void browseDataReconciliationCorrelationMatrixInputFile();
+  void showVariableFilterHelp();
   void simulationStarted();
   void simulationPaused();
 };
 
+class DataReconciliationDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  explicit DataReconciliationDialog(LibraryTreeItem *pLibraryTreeItem, QDialog *parent = nullptr);
+private:
+  LibraryTreeItem *mpLibraryTreeItem;
+  Label *mpDataReconciliationAlgorithmLabel;
+  QComboBox *mpDataReconciliationAlgorithmComboBox;
+  Label *mpDataReconciliationMeasurementInputFileLabel;
+  QLineEdit *mpDataReconciliationMeasurementInputFileTextBox;
+  QPushButton *mpDataReconciliationMeasurementInputFileBrowseButton;
+  Label *mpDataReconciliationCorrelationMatrixInputFileLabel;
+  QLineEdit *mpDataReconciliationCorrelationMatrixInputFileTextBox;
+  QPushButton *mpDataReconciliationCorrelationMatrixInputFileBrowseButton;
+
+  Label *mpBoundaryConditionMeasurementInputFileLabel;
+  QLineEdit *mpBoundaryConditionMeasurementInputFileTextBox;
+  QPushButton *mpBoundaryConditionMeasurementInputFileBrowseButton;
+  Label *mpBoundaryConditionCorrelationMatrixInputFileLabel;
+  QLineEdit *mpBoundaryConditionCorrelationMatrixInputFileTextBox;
+  QPushButton *mpBoundaryConditionCorrelationMatrixInputFileBrowseButton;
+
+  Label *mpDataReconciliationEpsilonLabel;
+  QLineEdit *mpDataReconciliationEpsilonTextBox;
+  QCheckBox *mpSaveSettingsCheckBox;
+  QPushButton *mpCalculateButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
+  QStackedWidget  *mpDataReconciliationStackedWidget;
+private slots:
+  void browseDataReconciliationMeasurementInputFile();
+  void browseDataReconciliationCorrelationMatrixInputFile();
+  void browseBoundaryConditionMeasurementInputFile();
+  void browseBoundaryConditionCorrelationMatrixInputFile();
+  void calculateDataReconciliation();
+  void switchAlgorithmPage(int index);
+};
+
 #endif // SIMULATIONDIALOG_H
