@@ -639,15 +639,12 @@ qreal Utilities::convertUnit(qreal value, qreal offset, qreal scaleFactor)
  */
 bool Utilities::isValueLiteralConstant(QString value)
 {
-  bool ok = true;
-  value.toDouble(&ok);
-  if (ok) return true;
-
-  QStringList valuesArray = StringHandler::removeFirstLastCurlBrackets(value).split(",");
-  foreach (QString valueElement, valuesArray) {
-    valueElement.toDouble(&ok);
-  }
-  return ok;
+  /* Issue #11795. Allow setting negative values for parameters.
+   * Issue #11840. Allow setting array of values.
+   * The following regular expression allows decimal values and array of decimal values. The values can be negative.
+   */
+  QRegExp rx("\\{?\\s*-?\\d+(\\.\\d+)?(?:\\s*,\\s*-?\\d+(\\.\\d+)?)*\\s*\\}?");
+  return rx.exactMatch(value);
 }
 
 /*!
