@@ -1,20 +1,21 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-CurrentYear, Linkoping University,
- * Department of Computer and Information Science,
- * SE-58183 Linkoping, Sweden.
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
+ * c/o Linköpings universitet, Department of Computer and Information Science,
+ * SE-58183 Linköping, Sweden.
  *
  * All rights reserved.
  *
- * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3
- * AND THIS OSMC PUBLIC LICENSE (OSMC-PL).
- * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES RECIPIENT'S
- * ACCEPTANCE OF THE OSMC PUBLIC LICENSE.
+ * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3 LICENSE OR
+ * THIS OSMC PUBLIC LICENSE (OSMC-PL) VERSION 1.2.
+ * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES
+ * RECIPIENT'S ACCEPTANCE OF THE OSMC PUBLIC LICENSE OR THE GPL VERSION 3,
+ * ACCORDING TO RECIPIENTS CHOICE.
  *
  * The OpenModelica software and the Open Source Modelica
  * Consortium (OSMC) Public License (OSMC-PL) are obtained
- * from Linkoping University, either from the above address,
+ * from OSMC, either from the above address,
  * from the URLs: http://www.ida.liu.se/projects/OpenModelica or
  * http://www.openmodelica.org, and in the OpenModelica distribution.
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
@@ -22,13 +23,10 @@
  * This program is distributed WITHOUT ANY WARRANTY; without
  * even the implied warranty of  MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
- * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS
- * OF OSMC-PL.
+ * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
  * See the full OSMC Public License conditions for more details.
  *
- * Main Author 2011: Adeel Asghar
- *
  */
 
 #include "PlotCurve.h"
@@ -41,21 +39,24 @@
 
 using namespace OMPlot;
 
-PlotCurve::PlotCurve(const QString &fileName, const QString &absoluteFilePath, const QString &name, const QString &xVariableName, const QString &yVariableName,
-                     const QString &unit, const QString &displayUnit, Plot *pParent)
+PlotCurve::PlotCurve(const QString &fileName, const QString &absoluteFilePath, const QString &xVariableName, const QString &xUnit, const QString &xDisplayUnit,
+                     const QString &yVariableName, const QString &yUnit, const QString &yDisplayUnit, Plot *pParent)
   : mCustomColor(false)
 {
-  mName = name;
+  mpParentPlot = pParent;
   mXVariable = xVariableName;
   mYVariable = yVariableName;
-  mNameStructure = fileName + "." + name;
+  mNameStructure = fileName + "." + yVariableName;
   mFileName = fileName;
   mAbsoluteFilePath = absoluteFilePath;
   mCustomColor = false;
-  setUnit(unit);
-  setDisplayUnit(displayUnit);
+  setXUnit(xUnit);
+  setXDisplayUnit(xDisplayUnit);
+  setYUnit(yUnit);
+  setYDisplayUnit(yDisplayUnit);
+  mCustomTitle = "";
+  setToggleSign(false);
   setTitleLocal();
-  mpParentPlot = pParent;
   /* set curve width and style */
   setCurveWidth(mpParentPlot->getParentPlotWindow()->getCurveWidth());
   setCurveStyle(mpParentPlot->getParentPlotWindow()->getCurveStyle());
@@ -72,10 +73,36 @@ PlotCurve::PlotCurve(const QString &fileName, const QString &absoluteFilePath, c
 
 void PlotCurve::setTitleLocal()
 {
-  if (getDisplayUnit().isEmpty()) {
-    QwtPlotItem::setTitle(getName());
+  if (mCustomTitle.isEmpty()) {
+    QString titleStr = getYVariable();
+    if (!getYDisplayUnit().isEmpty() || !mpParentPlot->getYScaleDraw()->getUnitPrefix().isEmpty()) {
+      titleStr += QString(" (%1%2)").arg(mpParentPlot->getYScaleDraw()->getUnitPrefix(), getYDisplayUnit());
+    }
+
+    if (mpParentPlot->getParentPlotWindow()->getPlotType() == PlotWindow::PLOTPARAMETRIC) {
+      QString xVariable = getXVariable();
+      if (!getXDisplayUnit().isEmpty() || !mpParentPlot->getXScaleDraw()->getUnitPrefix().isEmpty()) {
+        xVariable += QString(" (%1%2)").arg(mpParentPlot->getXScaleDraw()->getUnitPrefix(), getXDisplayUnit());
+      }
+      if (!xVariable.isEmpty()) {
+        titleStr += QString(" <b>vs</b> %1").arg(xVariable);
+      }
+    }
+    // Add - sign if curve is toggled
+    if (getToggleSign()) {
+      titleStr.prepend(QString("-"));
+    }
+    setTitle(titleStr);
+    // visibility
+    QwtText text = title();
+    if (isVisible()) {
+      text.setColor(QColor(Qt::black));
+    } else {
+      text.setColor(QColor(Qt::gray));
+    }
+    setTitle(text);
   } else {
-    QwtPlotItem::setTitle(QString("%1 (%2)").arg(getName(), getDisplayUnit()));
+    setTitle(mCustomTitle);
   }
 }
 
@@ -232,16 +259,9 @@ bool PlotCurve::hasCustomColor()
  * \brief PlotCurve::toggleVisibility
  * Toggles the curve visibility.
  */
-void PlotCurve::toggleVisibility()
+void PlotCurve::toggleVisibility(bool visibility)
 {
-  setVisible(!isVisible());
-  QwtText text = title();
-  if (isVisible()) {
-    text.setColor(QColor(Qt::black));
-  } else {
-    text.setColor(QColor(Qt::gray));
-  }
-  setTitle(text);
+  setVisible(visibility);
 }
 
 void PlotCurve::setData(const double* xData, const double* yData, int size)
@@ -315,13 +335,23 @@ int PlotCurve::closestPoint(const QPoint &pos, double *dist) const
  */
 QRectF PlotCurve::boundingRect() const
 {
-  QRectF r = QwtPlotCurve::boundingRect();
-  if (r.isValid()) {
+  QRectF existingBoundingRect = QwtPlotCurve::boundingRect();
+  if (existingBoundingRect.isValid()) {
     /* Ticket:5515 Allow for some margin at the top and bottom of plot windows in OMEdit
      * We add a margin of 10% to the curves bottom and top to better visualize them.
      */
-    const double margin = r.height() * 0.1;
-    r.adjust(0, -margin, 0, margin);
+    const double margin = existingBoundingRect.height() * 0.1;
+    QRectF newBoundingRect = existingBoundingRect.adjusted(0, -margin, 0, margin);
+
+    /* Issue:7468 Check for negative values doing log plot.
+     * Do not add margin if the value becomes negative.
+     */
+    if ((mpParentPlot->getParentPlotWindow()->getLogXCheckBox()->isChecked() || mpParentPlot->getParentPlotWindow()->getLogYCheckBox()->isChecked())
+        && (newBoundingRect.top() < 0.0 || newBoundingRect.bottom() < 0.0)) {
+      return existingBoundingRect;
+    } else {
+      return newBoundingRect;
+    }
   }
-  return r;
+  return existingBoundingRect;
 }
